@@ -11,14 +11,19 @@ from django.utils.dateparse import parse_datetime
 
 from allianceauth.services.hooks import get_extension_logger
 
-from .. import __title__
+from app_utils.json import JSONDateTimeDecoder, JSONDateTimeEncoder
+from app_utils.logging import LoggerAddTag
+
+from .. import __title__, USER_AGENT_TEXT
+from ..app_settings import KILLTRACKER_REDISQ_TTW, KILLTRACKER_REDISQ_LOCK_TIMEOUT
 from ..providers import esi
-from ..utils import LoggerAddTag, JsonDateTimeDecoder, JsonDateTimeEncoder
+
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
 ZKB_REDISQ_URL = "https://redisq.zkillboard.com/listen.php"
 ZKB_API_URL = "https://zkillboard.com/api/"
+ZKB_KILLMAIL_BASEURL = "https://zkillboard.com/kill/"
 REQUESTS_TIMEOUT = (5, 30)
 
 
@@ -112,6 +117,8 @@ class TrackerInfo(_KillmailBase):
 
 @dataclass
 class Killmail(_KillmailBase):
+    HTTP_TOO_MANY_REQUESTS = 429
+
     id: int
     time: datetime
     victim: KillmailVictim
@@ -121,6 +128,9 @@ class Killmail(_KillmailBase):
     solar_system_id: Optional[int] = None
     tracker_info: Optional[TrackerInfo] = None
 
+    def __repr__(self):
+        return f"{type(self).__name__}(id={self.id})"
+
     def attackers_alliance_ids(self) -> List[int]:
         """returns alliance IDs of all attackers"""
         return [
@@ -180,22 +190,43 @@ class Killmail(_KillmailBase):
             raise ex
 
     def asjson(self) -> str:
-        return json.dumps(asdict(self), cls=JsonDateTimeEncoder)
+        return json.dumps(asdict(self), cls=JSONDateTimeEncoder)
 
     @classmethod
     def from_json(cls, json_str: str) -> "Killmail":
-        return cls.from_dict(json.loads(json_str, cls=JsonDateTimeDecoder))
+        return cls.from_dict(json.loads(json_str, cls=JSONDateTimeDecoder))
 
     @classmethod
-    def create_from_zkb_redisq(cls) -> "Killmail":
+    def create_from_zkb_redisq(cls) -> Optional["Killmail"]:
         """Fetches and returns a killmail from ZKB.
 
         Returns None if no killmail is received.
         """
         logger.info("Trying to fetch killmail from ZKB RedisQ...")
-        r = requests.get(ZKB_REDISQ_URL, timeout=REQUESTS_TIMEOUT)
+        lock_key = f"{__title__.upper()}_REDISQ_LOCK"
+        lock = cache.lock(key=lock_key, timeout=KILLTRACKER_REDISQ_LOCK_TIMEOUT)
+        acquired = lock.acquire(blocking=False)
+        if not acquired:
+            logger.warning("Failed to acquire lock for atomic access to RedisQ.")
+            return None
+        r = requests.get(
+            ZKB_REDISQ_URL,
+            params={"ttw": KILLTRACKER_REDISQ_TTW},
+            timeout=REQUESTS_TIMEOUT,
+            headers={"User-Agent": USER_AGENT_TEXT},
+        )
+        lock.release()
+        if r.status_code == cls.HTTP_TOO_MANY_REQUESTS:
+            logger.error("429 Client Error: Too many requests: %s", r.text)
+            return None
         r.raise_for_status()
-        data = r.json()
+        try:
+            data = r.json()
+        except Exception:
+            logger.warning(
+                "ZKB did not return JSON as was expected: %s", r.text, exc_info=True
+            )
+            return None
         if data:
             logger.debug("data:\n%s", data)
         if data and "package" in data and data["package"]:
@@ -207,12 +238,11 @@ class Killmail(_KillmailBase):
             return None
 
     @classmethod
-    def create_from_zkb_api(cls, killmail_id: int) -> "Killmail":
+    def create_from_zkb_api(cls, killmail_id: int) -> Optional["Killmail"]:
         """Fetches and returns a killmail from ZKB API.
 
         results are cached
         """
-
         cache_key = f"{__title__.upper()}_KILLMAIL_{killmail_id}"
         killmail_json = cache.get(cache_key)
         if killmail_json:
@@ -223,7 +253,9 @@ class Killmail(_KillmailBase):
                 killmail_id,
             )
             url = f"{ZKB_API_URL}killID/{killmail_id}/"
-            r = requests.get(url, timeout=REQUESTS_TIMEOUT)
+            r = requests.get(
+                url, timeout=REQUESTS_TIMEOUT, headers={"User-Agent": USER_AGENT_TEXT}
+            )
             r.raise_for_status()
             zkb_data = r.json()
             if not zkb_data:
@@ -264,6 +296,9 @@ class Killmail(_KillmailBase):
 
     @staticmethod
     def _create_from_dict(package_data: dict) -> "Killmail":
+        """creates a new object from given dict.
+        Needs to confirm with data structure returned from ZKB RedisQ
+        """
         zkb = KillmailZkb()
         if "zkb" in package_data:
             zkb_data = package_data["zkb"]
