@@ -1,4 +1,5 @@
 from datetime import timedelta
+
 from typing import Dict, Tuple
 
 from django.db import models, transaction
@@ -6,12 +7,13 @@ from django.utils.timezone import now
 
 from allianceauth.services.hooks import get_extension_logger
 
+from app_utils.caching import ObjectCacheMixin
+from app_utils.logging import LoggerAddTag
 from eveuniverse.models import EveEntity
 
 from . import __title__
 from .app_settings import KILLTRACKER_PURGE_KILLMAILS_AFTER_DAYS
 from .core.killmails import Killmail, _KillmailCharacter
-from .utils import LoggerAddTag
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
@@ -42,8 +44,15 @@ class EveKillmailManager(models.Manager):
             deadline = now() - timedelta(days=KILLTRACKER_PURGE_KILLMAILS_AFTER_DAYS)
             return self.filter(time__lt=deadline).delete()
 
-    def create_from_killmail(self, killmail: Killmail) -> models.Model:
-        """create a new EveKillmail from a Killmail object and returns it"""
+    def create_from_killmail(
+        self, killmail: Killmail, resolve_ids=True
+    ) -> models.Model:
+        """create a new EveKillmail from a Killmail object and returns it
+
+        Args:
+        - resolve_ids: When set to False will not resolve EveEntity IDs
+
+        """
         from .models import (
             EveKillmailAttacker,
             EveKillmailPosition,
@@ -51,44 +60,46 @@ class EveKillmailManager(models.Manager):
             EveKillmailZkb,
         )
 
-        with transaction.atomic():
-            args = {
-                "id": killmail.id,
-                "time": killmail.time,
-            }
-            if killmail.solar_system_id:
-                args["solar_system"], _ = EveEntity.objects.get_or_create(
-                    id=killmail.solar_system_id
-                )
-            eve_killmail = self.create(**args)
+        args = {
+            "id": killmail.id,
+            "time": killmail.time,
+        }
+        if killmail.solar_system_id:
+            args["solar_system"], _ = EveEntity.objects.get_or_create(
+                id=killmail.solar_system_id
+            )
+        eve_killmail = self.create(**args)
 
-            if killmail.zkb:
-                args = {**killmail.zkb.asdict(), **{"killmail": eve_killmail}}
-                EveKillmailZkb.objects.create(**args)
+        if killmail.zkb:
+            args = {**killmail.zkb.asdict(), **{"killmail": eve_killmail}}
+            EveKillmailZkb.objects.create(**args)
 
+        args = {
+            **{
+                "killmail": eve_killmail,
+                "damage_taken": killmail.victim.damage_taken,
+            },
+            **self._create_args_for_entities(killmail.victim),
+        }
+        EveKillmailVictim.objects.create(**args)
+
+        args = {**killmail.position.asdict(), **{"killmail": eve_killmail}}
+        EveKillmailPosition.objects.create(**args)
+
+        for attacker in killmail.attackers:
             args = {
                 **{
                     "killmail": eve_killmail,
-                    "damage_taken": killmail.victim.damage_taken,
+                    "damage_done": attacker.damage_done,
+                    "security_status": attacker.security_status,
+                    "is_final_blow": attacker.is_final_blow,
                 },
-                **self._create_args_for_entities(killmail.victim),
+                **self._create_args_for_entities(attacker),
             }
-            EveKillmailVictim.objects.create(**args)
+            EveKillmailAttacker.objects.create(**args)
 
-            args = {**killmail.position.asdict(), **{"killmail": eve_killmail}}
-            EveKillmailPosition.objects.create(**args)
-
-            for attacker in killmail.attackers:
-                args = {
-                    **{
-                        "killmail": eve_killmail,
-                        "damage_done": attacker.damage_done,
-                        "security_status": attacker.security_status,
-                        "is_final_blow": attacker.is_final_blow,
-                    },
-                    **self._create_args_for_entities(attacker),
-                }
-                EveKillmailAttacker.objects.create(**args)
+        if resolve_ids:
+            EveEntity.objects.bulk_update_new_esi()
 
         eve_killmail.refresh_from_db()
         return eve_killmail
@@ -114,6 +125,17 @@ class EveKillmailManager(models.Manager):
             except self.model.DoesNotExist:
                 created = True
 
-            obj = self.create_from_killmail(killmail)
+            obj = self.create_from_killmail(killmail, resolve_ids=False)
+
+        if created:
+            EveEntity.objects.bulk_update_new_esi()
 
         return obj, created
+
+
+class TrackerManager(ObjectCacheMixin, models.Manager):
+    pass
+
+
+class WebhookManager(ObjectCacheMixin, models.Manager):
+    pass
